import os, sys, shutil, gzip
sys.path.append( "./src" )

def copyfile( srcfile, dstfile ):
    dstdir = os.path.split(dstfile)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    shutil.copy( srcfile, dstfile )

def copyfiles( srcdir, dstdir ):
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    for file in os.listdir( srcdir ):
        if not os.path.isdir( srcdir + "/" + file ):
            shutil.copy( srcdir + "/" + file, dstdir + "/" + file )

def copygzip( srcfile, dstfile ):
    os.system("gzip "+srcfile )
    shutil.copy( srcfile+".gz", dstfile )
    os.system("gzip -d "+srcfile+".gz")
    
def copypacks( repo_root, dstdir ):
    for wdifile in os.listdir( repo_root ): 
        if not wdifile.endswith(".wdi"): 
            continue
        shutil.copy( repo_root + wdifile, dstdir + wdifile )

def usage( error=None ):
    if error:
        print "Error: " + error
    print "Usage: python setup.py [command] <options>"
    print " Commands:"
    print "   install - installs wine-doors"
    print " uninstall - uninstalls wine-doors"
    print " remove-all - Remove every wine-doors previews version (only use if you know what you are doing)"
    print "    --help - display this usage message (default)\n"
    print " Options:"
    print "  --prefix - prefix path default ~/.local/ (ex: /)"
    print "  --root - set the install root usuall / or ~/ used for packaging"
    sys.exit(0)
    pass
    
def symlink( srclink, dstlink ):
    dstdir = os.path.split(dstlink)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    if not os.path.islink( dstlink ):
        os.symlink( srclink, dstlink )

if __name__ == "__main__":
    os.chdir(sys.path[0])
    wine_doors = "wine-doors"
    command = None
    sysinstall = False    
    install_root = None
    prefix = None

    # Read command line arguments
    for argument in sys.argv:
        if argument == "install" or argument == "uninstall" or argument == "remove-all":
            command = argument
        elif argument.startswith( "--temp" ):
            temp = argument.split( "=" )[1]
        elif argument.startswith( "--prefix" ):
            prefix = argument.split( "=" )[1]
        elif argument.startswith( "--root" ):
            install_root = argument.split( "=" )[1]
            if not install_root.endswith("/"):
                install_root = install_root + "/"
        elif argument.startswith( "--sysinstall"):
            sysinstall = True
        elif argument.startswith( "--help" ):
            usage()

    if os.getuid() == 0:
        sysinstall = True

    if sysinstall:
        if not install_root:
            install_root = "/"
        if not prefix:
            prefix = install_root + "usr/"
        bin_path = prefix + "bin/"
        data_path = prefix + "share/"
        conf_path = install_root + "etc/" + wine_doors + "/"
        winedoors_path = data_path + wine_doors + "/"
        real_winedoors_path = "/usr/share/"+wine_doors+"/"
    else:
        install_root = os.path.expanduser( "~/")
        prefix = os.path.expanduser( "~/.local" )
        bin_path = install_root + "bin/"
        data_path = prefix + "share/"
        conf_path = os.getenv('WINEPREFIX', install_root + ".wine") + "/wine-doors"
        winedoors_path = data_path + wine_doors + "/"
        real_winedoors_path = winedoors_path

    if command in ("install", "uninstall"):
        from preferences import preferences
        
    if command == "install":
        depsmet = True
        hasui = True
        print "Checking dependencies"
        print "  wine . . . ",
        if os.popen( "which wine" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
            depsmet = False
        print "  cabextract . . . ",
        if os.popen( "which cabextract" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
            depsmet = False
        print "  tar . . . ",
        if os.popen( "which tar" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
            depsmet = False
        print "  ps . . . ",
        if os.popen( "which ps" ).read().rstrip() != "":
            print "Found"
        else:
            print "Not found !"
            depsmet = False
            
        print "  pygtk . . . (python-gtk2)",
        try:
            import gtk, gobject
            print "Found"
        except:
            print "Not found"
            hasui = False

        print "  pycairo . . . (python-cairo)",
        try:
            import cairo
            print "Found"
        except:
            print "Not found"
            hasui = False

        print "  rsvg . . . (python-gnome2-desktop)",
        try:
            import rsvg
            print "Found"
        except:
            print "Not found"
            hasui = False

        if not depsmet:
            print "Dependencies not met"
            sys.exit(0)

        if not hasui:
            print "One of the required modules for the wine-doors user interface is missing"
            print "the wine-doors command line interface is available, for more information" 
            print "try running: wine-doors --help"

        if sysinstall:
            print "Performing system install"

        #Create directories
        if not os.path.isdir( data_path ):
            os.makedirs( data_path )
        if not os.path.isdir( winedoors_path ):
            os.makedirs( winedoors_path )
        if not os.path.isdir( bin_path ):
            os.makedirs( bin_path )
        if not os.path.isdir( data_path + "pixmaps/" ):
            os.makedirs( data_path + "pixmaps/" )
        if not os.path.isdir( conf_path ):
            os.makedirs( conf_path )

        conf_file = conf_path + "/preferences.xml"

        # Install sources
        print "Installing wine-doors"
        copyfiles( "./src", winedoors_path + "src" )
        os.system( "chmod +x " + winedoors_path + "src/winedoors.py" )
        copyfiles( "./pixmaps", winedoors_path + "pixmaps/" )
        copyfiles( "./dtd", winedoors_path + "dtd/" )
        copyfiles( "./registry", winedoors_path + "registry/" )

        print "Installing desktop launcher"
        copyfile( "./dist/wine-doors.desktop",  data_path + "/applications/wine-doors.desktop" )
        if os.path.isfile( data_path + "/applications/wine-doors.desktop" ):
            desktop = open( data_path + "/applications/wine-doors.desktop" , 'a' )
            if sysinstall:
                desktop.write( 'Exec=wine-doors\n' )
                desktop.write( 'Icon=wine-doors\n' )
            else:
                desktop.write( 'Exec=' + bin_path + "wine-doors\n" )
                desktop.write( 'Icon=' + real_winedoors_path + "pixmaps/wine-doors.png\n" )
            desktop.close()

        # install symlinks
        if sysinstall:
            print "Symlinking icons"
            symlink( real_winedoors_path + "pixmaps/wine-doors.png", data_path + "/pixmaps/wine-doors.png" )
            symlink( real_winedoors_path + "pixmaps/wine-doors.svg", data_path + "/pixmaps/wine-doors.svg" )
        
        print "Symlinking executable"
        symlink( real_winedoors_path + "src/winedoors.py", bin_path + "/wine-doors" )
        
        # Fix perms
        if sysinstall:
            print "Setting permissions"
            os.system( "chmod og+r -R "+winedoors_path )
        
        # Write preferences.xml
        print "Creating initial preferences"
        preferences.Default()
        preferences.preferences['prefix'] = real_winedoors_path
        preferences.preferences['wineroot'] = ""
        preferences.AddRepo( "Applications", "http://www.wine-doors.org/repositories/applications.repo/" )
        preferences.AddRepo( "System Base", "http://www.wine-doors.org/repositories/base.repo/" )
        preferences.AddRepo( "Games", "http://www.wine-doors.org/repositories/games.repo/" )
        preferences.AddRepo( "Libraries and Fonts", "http://www.wine-doors.org/repositories/libraries.repo/" )
        preferences.Save( conf_file )
        
    elif command == "uninstall":
        if sysinstall:
            print "Removing system wide preferences"
            if os.path.exists( conf_path ):
                shutil.rmtree( conf_path )
        
        print "Removing symlinks"
        
        if sysinstall:
            if os.path.exists( data_path + "pixmaps/wine-doors.png" ):
                os.remove( data_path + "pixmaps/wine-doors.png" )
            if os.path.exists( data_path + "pixmaps/wine-doors.svg" ):
                os.remove( data_path + "pixmaps/wine-doors.svg" )

        if os.path.exists( bin_path + "wine-doors" ):
            os.remove( bin_path + "wine-doors" )
            
        print "Removing menu item"
        if os.path.exists( data_path + "applications/wine-doors.desktop" ):
            os.remove ( data_path + "applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( winedoors_path ):
            shutil.rmtree( winedoors_path, True)
            
    elif command == "remove-all":
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" ))
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" ))

        if os.path.exists( os.path.expanduser( "~/.wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.wine-doors" ), True )

        if os.path.exists( os.getenv('WINEPREFIX', os.path.expanduser( "~/.wine" ))):
            shutil.rmtree( os.getenv('WINEPREFIX', os.path.expanduser( "~/.wine" )), True )
            
        print "Removing menu item"
        if os.path.exists( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" )):
            os.remove ( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" ))
        if os.path.isdir( os.path.expanduser( "~/.local/share/applications/wine" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/applications/wine" ))
        print "Removing shared data"
        if os.path.isdir( os.path.expanduser( "~/.local/share/wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/wine-doors" ), True)

        # Removing root install
        if os.path.exists( "/usr/share/pixmaps/wine-doors.png" ):
            os.remove( "/usr/share/pixmaps/wine-doors.png" )
        if os.path.exists( "/usr/share/pixmaps/wine-doors.svg" ):
            os.remove( "/usr/share/pixmaps/wine-doors.svg" )

        if os.path.isdir( "/etc/wine-doors" ):
            shutil.rmtree( "/etc/wine-doors", True )

        print "Removing menu item"
        if os.path.exists( "/usr/share/applications/wine-doors.desktop" ):
            os.remove ( "/usr/share/applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( "/usr/share/wine-doors" ):
            shutil.rmtree( "/usr/share/wine-doors", True)

    else:
        usage("No command specified")
